@@ -57,18 +57,16 @@ sub authenticate {
                 "  logincookies.userid=profiles.userid AND " .
                 "  logincookies.userid=? AND " .
                 "  (logincookies.ipaddr=?";
+    my @params = ($login_cookie, $login, $ipaddr);
     if (defined $netaddr) {
         trick_taint($netaddr);
         $query .= " OR logincookies.ipaddr=?";
+        push(@params, $netaddr);
     }
     $query .= ")";
 
     my $dbh = Bugzilla->dbh;
-    my ($userid, $disabledtext) = $dbh->selectrow_array($query, undef,
-                                                        $login_cookie,
-                                                        $login,
-                                                        $ipaddr,
-                                                        $netaddr);
+    my ($userid, $disabledtext) = $dbh->selectrow_array($query, undef, @params);
 
     return (AUTH_DISABLED, $userid, $disabledtext)
       if ($disabledtext);
@@ -80,10 +78,6 @@ sub authenticate {
                  undef,
                  $login_cookie);
 
-        # compat code. The cookie value is used for logouts, and that
-        # isn't generic yet. Detaint it so that its usable
-        detaint_natural($::COOKIE{'Bugzilla_logincookie'});
-
         return (AUTH_OK, $userid);
     }
 
