@@ -30,7 +30,7 @@ use strict;
 
 package Bugzilla::BugMail;
 
-use RelationSet;
+use Bugzilla::RelationSet;
 
 use Bugzilla::Config qw(:DEFAULT $datadir);
 use Bugzilla::Util;
@@ -166,7 +166,7 @@ sub ProcessOneBug($) {
     trick_taint($start);
     trick_taint($end);
 
-    my $ccSet = new RelationSet();
+    my $ccSet = new Bugzilla::RelationSet();
     $ccSet->mergeFromDB("SELECT who FROM cc WHERE bug_id = $id");
     $values{'cc'} = $ccSet->toString();
     
@@ -221,12 +221,11 @@ sub ProcessOneBug($) {
 
     my $difftext = "";
     my $diffheader = "";
-    my $diffpart = {};
     my @diffparts;
     my $lastwho = "";
     foreach my $ref (@diffs) {
         my ($who, $what, $when, $old, $new, $attachid, $fieldname) = (@$ref);
-        $diffpart = {};
+        my $diffpart = {};
         if ($who ne $lastwho) {
             $lastwho = $who;
             $diffheader = "\n$who" . Param('emailsuffix') . " changed:\n\n";
@@ -239,6 +238,11 @@ sub ProcessOneBug($) {
             $old = FormatTimeUnit($old);
             $new = FormatTimeUnit($new);
         }
+        if ($attachid) {
+            SendSQL("SELECT isprivate FROM attachments 
+                     WHERE attach_id = $attachid");
+            $diffpart->{'isprivate'} = FetchOneColumn();
+        }
         $difftext = FormatTriple($what, $old, $new);
         $diffpart->{'header'} = $diffheader;
         $diffpart->{'fieldname'} = $fieldname;
@@ -249,8 +253,6 @@ sub ProcessOneBug($) {
 
     my $deptext = "";
 
-    my $resid = 
-
     SendSQL("SELECT bugs_activity.bug_id, bugs.short_desc, fielddefs.name, " .
             "       removed, added " .
             "FROM bugs_activity, bugs, dependencies, fielddefs ".
@@ -301,8 +303,8 @@ sub ProcessOneBug($) {
     $deptext = trim($deptext);
 
     if ($deptext) {
-        #$difftext = trim($difftext . "\n\n" . $deptext);
-        $diffpart->{'text'} = trim("\n\n" . $deptext);
+        my $diffpart = {};
+        $diffpart->{'text'} = "\n" . trim("\n\n" . $deptext);
         push(@diffparts, $diffpart);
     }
 
@@ -613,15 +615,6 @@ sub filterEmailGroup ($$$) {
         SendSQL("SELECT emailflags FROM profiles WHERE userid = $userid");
         my $prefs = FetchOneColumn();
         
-        # If the user's preferences are empty, it means the user has not set
-        # their mail preferences after the installation upgraded from a
-        # version of Bugzilla without email preferences to one with them. In
-        # this case, assume they want to receive all mail.
-        if (!defined($prefs) || $prefs !~ /email/) {
-            push(@recipients, $user);
-            next;
-        }
-        
         # Write the user's preferences into a Perl record indexed by 
         # preference name.  We pass the value "255" to the split function 
         # because otherwise split will trim trailing null fields, causing 
@@ -663,10 +656,7 @@ sub filterEmailGroup ($$$) {
         }
         
         # If the user prefers to be included in mail about this change,
-        # or they haven't specified a preference for it (because they
-        # haven't visited the email preferences page since the preference
-        # was added, in which case we include them by default), add them
-        # to the list of recipients.
+        # add them to the list of recipients.
         foreach my $reason (@$reasons) {
             my $pref = "email$role$reason";
             if (!exists($prefs{$pref}) || $prefs{$pref} eq 'on') {
@@ -708,6 +698,9 @@ sub NewProcessOnePerson ($$$$$$$$$$$$$) {
     # This routine should really get passed a userid
     # This rederives groups as a side effect
     my $user = Bugzilla::User->new_from_login($person);
+    if (!$user) { # person doesn't exist, probably changed email
+      return;
+    }
     my $userid = $user->id;
 
     $seen{$person} = 1;
@@ -772,6 +765,11 @@ sub NewProcessOnePerson ($$$$$$$$$$$$$) {
             if ($user->groups->{Param("timetrackinggroup")}) {
                 $add_diff = 1;
             }
+        } elsif (($diff->{'isprivate'}) 
+                 && Param('insidergroup')
+                 && !($user->groups->{Param('insidergroup')})
+                ) {
+            $add_diff = 0;
         } else {
             $add_diff = 1;
         }
@@ -852,20 +850,27 @@ sub NewProcessOnePerson ($$$$$$$$$$$$$) {
     
     my $msg = PerformSubsts($template, \%substs);
 
-    my $sendmailparam = "-ODeliveryMode=deferred";
-    if (Param("sendmailnow")) {
-       $sendmailparam = "";
+    MessageToMTA($msg, $person);
+
+    push(@sentlist, $person);
+    return 1;
+}
+
+sub MessageToMTA ($;$) {
+   my ($msg, $rcpt_to) = (@_);
+
+    my $sendmailparam = "";
+    unless (Param("sendmailnow")) {
+       $sendmailparam = "-ODeliveryMode=deferred";
     }
 
     if ($enableSendMail == 1) {
         open(SENDMAIL, "|/usr/lib/sendmail $sendmailparam -t -i") ||
           die "Can't open sendmail";
-    
+
         print SENDMAIL trim($msg) . "\n";
         close SENDMAIL;
     }
-    push(@sentlist, $person);
-    return 1;
 }
 
 1;
