@@ -33,7 +33,7 @@ use base qw(Exporter);
 
 %Bugzilla::DB::EXPORT_TAGS =
   (
-   deprecated => [qw(ConnectToDatabase SendSQL SqlQuote
+   deprecated => [qw(SendSQL SqlQuote
                      MoreSQLData FetchSQLData FetchOneColumn
                      PushGlobalSQLState PopGlobalSQLState)
                  ],
@@ -49,15 +49,14 @@ use Bugzilla::Util;
 # having a separate package for it, or otherwise trying to avoid the circular
 # dependancy
 
-sub ConnectToDatabase {
-    # We've already been connected in Bugzilla.pm
-}
-
 # XXX - mod_perl
 # These use |our| instead of |my| because they need to be cleared from
 # Bugzilla.pm. See bug 192531 for details.
 our $_current_sth;
 our @SQLStateStack = ();
+
+my $_fetchahead;
+
 sub SendSQL {
     my ($str) = @_;
 
@@ -68,6 +67,8 @@ sub SendSQL {
     # This is really really ugly, but its what we get for not doing
     # error checking for 5 years. See bug 189446 and bug 192531
     $_current_sth->{RaiseError} = 0;
+
+    undef $_fetchahead;
 }
 
 # Its much much better to use bound params instead of this
@@ -84,8 +85,6 @@ sub SqlQuote {
     return $res;
 }
 
-# XXX - mod_perl
-my $_fetchahead;
 sub MoreSQLData {
     return 1 if defined $_fetchahead;
 
@@ -148,14 +147,18 @@ sub _connect {
     # connect using our known info to the specified db
     # Apache::DBI will cache this when using mod_perl
     my $dbh = DBI->connect($dsn,
-                           $db_user,
-                           $db_pass,
+                           '',
+                           '',
                            { RaiseError => 1,
                              PrintError => 0,
+                             Username => $db_user,
+                             Password => $db_pass,
                              ShowErrorStatement => 1,
                              HandleError => \&_handle_error,
-                             FetchHashKeyName => 'NAME_lc',
                              TaintIn => 1,
+                             FetchHashKeyName => 'NAME',  
+                             # Note: NAME_lc causes crash on ActiveState Perl
+                             # 5.8.4 (see Bug 253696)
                            });
 
     return $dbh;
@@ -164,10 +167,23 @@ sub _connect {
 sub _handle_error {
     require Carp;
 
+    # Cut down the error string to a reasonable size
+    $_[0] = substr($_[0], 0, 2000) . ' ... ' . substr($_[0], -2000)
+        if length($_[0]) > 4000;
     $_[0] = Carp::longmess($_[0]);
     return 0; # Now let DBI handle raising the error
 }
 
+my $cached_server_version;
+sub server_version {
+    return $cached_server_version if defined($cached_server_version);
+    my $dbh = Bugzilla->dbh;
+    my $sth = $dbh->prepare('SELECT VERSION()');
+    $sth->execute();
+    ($cached_server_version) = $sth->fetchrow_array();
+    return $cached_server_version;
+}
+
 1;
 
 __END__
@@ -222,10 +238,6 @@ and so are not documented.
 
 =item *
 
-ConnectToDatabase
-
-=item *
-
 SendSQL
 
 =item *
