@@ -89,26 +89,38 @@ sub get_exclusions {
     return get_clusions($id, "ex");
 }
 
+# Return a hash of product/component IDs and names
+# associated with the flagtype:
+# $clusions{'product_name:component_name'} = "product_ID:component_ID"
+
 sub get_clusions {
     my ($id, $type) = @_;
     
-    &::PushGlobalSQLState();
-    &::SendSQL("SELECT products.name, components.name " . 
-               "FROM flagtypes, flag${type}clusions " . 
-               "LEFT OUTER JOIN products ON flag${type}clusions.product_id = products.id " . 
-               "LEFT OUTER JOIN components ON flag${type}clusions.component_id = components.id " . 
-               "WHERE flagtypes.id = $id AND flag${type}clusions.type_id = flagtypes.id");
-    my @clusions = ();
-    while (&::MoreSQLData()) {
-        my ($product, $component) = &::FetchSQLData();
-        $product ||= "Any";
-        $component ||= "Any";
-        push(@clusions, "$product:$component");
-    }
-    &::PopGlobalSQLState();
+    my $dbh = Bugzilla->dbh;
+
+    my $list =
+        $dbh->selectall_arrayref("SELECT products.id, products.name, " .
+                                 "       components.id, components.name " . 
+                                 "FROM flagtypes, flag${type}clusions " . 
+                                 "LEFT OUTER JOIN products " .
+                                 "  ON flag${type}clusions.product_id = products.id " . 
+                                 "LEFT OUTER JOIN components " .
+                                 "  ON flag${type}clusions.component_id = components.id " . 
+                                 "WHERE flagtypes.id = ? " .
+                                 " AND flag${type}clusions.type_id = flagtypes.id",
+                                 undef, $id);
+    my %clusions;
+    foreach my $data (@$list) {
+        my ($product_id, $product_name, $component_id, $component_name) = @$data;
+        $product_id ||= 0;
+        $product_name ||= "__Any__";
+        $component_id ||= 0;
+        $component_name ||= "__Any__";
+        $clusions{"$product_name:$component_name"} = "$product_id:$component_id";
+     }
     
-    return \@clusions;
-}
+    return \%clusions;
+ }
 
 sub match {
     # Queries the database for flag types matching the given criteria
@@ -182,15 +194,28 @@ sub count {
 
 sub validate {
     my ($data, $bug_id, $attach_id) = @_;
-  
+    my $dbh = Bugzilla->dbh;
+
     # Get a list of flag types to validate.  Uses the "map" function
     # to extract flag type IDs from form field names by matching columns
     # whose name looks like "flag_type-nnn", where "nnn" is the ID,
     # and returning just the ID portion of matching field names.
     my @ids = map(/^flag_type-(\d+)$/ ? $1 : (), keys %$data);
-  
-    foreach my $id (@ids)
-    {
+
+    return unless scalar(@ids);
+
+    # No flag reference should exist when changing several bugs at once.
+    ThrowCodeError("flags_not_available") unless $bug_id;
+
+    # All flag types have to be active
+    my $inactive_flagtypes =
+        $dbh->selectrow_array("SELECT 1 FROM flagtypes
+                               WHERE id IN (" . join(',', @ids) . ")
+                               AND is_active = 0 LIMIT 1");
+
+    ThrowCodeError("flag_type_inactive") if $inactive_flagtypes;
+
+    foreach my $id (@ids) {
         my $status = $data->{"flag_type-$id"};
         
         # Don't bother validating types the user didn't touch.
@@ -211,23 +236,32 @@ sub validate {
             ThrowCodeError("flag_status_invalid", 
                            { id => $id , status => $status });
         }
-        
+
+        # Make sure the user didn't specify a requestee unless the flag
+        # is specifically requestable.
+        my $new_requestee = trim($data->{"requestee_type-$id"} || '');
+
+        if ($status eq '?'
+            && !$flag_type->{is_requesteeble}
+            && $new_requestee)
+        {
+            ThrowCodeError("flag_requestee_disabled",
+                           { name => $flag_type->{name} });
+        }
+
         # Make sure the requestee is authorized to access the bug
         # (and attachment, if this installation is using the "insider group"
         # feature and the attachment is marked private).
         if ($status eq '?'
             && $flag_type->{is_requesteeble}
-            && trim($data->{"requestee_type-$id"}))
+            && $new_requestee)
         {
-            my $requestee_email = trim($data->{"requestee_type-$id"});
-
             # We know the requestee exists because we ran
             # Bugzilla::User::match_field before getting here.
-            my $requestee = Bugzilla::User->new_from_login($requestee_email);
+            my $requestee = Bugzilla::User->new_from_login($new_requestee);
 
             # Throw an error if the user can't see the bug.
-            if (!&::CanSeeBug($bug_id, $requestee->id))
-            {
+            if (!&::CanSeeBug($bug_id, $requestee->id)) {
                 ThrowUserError("flag_requestee_unauthorized",
                                { flag_type => $flag_type,
                                  requestee => $requestee,
@@ -270,6 +304,7 @@ sub normalize {
             AND (bugs.product_id = i.product_id OR i.product_id IS NULL)
             AND (bugs.component_id = i.component_id OR i.component_id IS NULL))
         WHERE flags.type_id IN ($ids)
+        AND flags.is_active = 1
         AND i.type_id IS NULL
     ");
     Bugzilla::Flag::clear(&::FetchOneColumn()) while &::MoreSQLData();
@@ -280,6 +315,7 @@ sub normalize {
         WHERE flags.type_id IN ($ids)
         AND flags.bug_id = bugs.bug_id
         AND flags.type_id = e.type_id 
+        AND flags.is_active = 1
         AND (bugs.product_id = e.product_id OR e.product_id IS NULL)
         AND (bugs.component_id = e.component_id OR e.component_id IS NULL)
     ");
