@@ -32,6 +32,8 @@ use strict;
 use vars qw($userid);
 
 package Bugzilla::Search;
+use base qw(Exporter);
+@Bugzilla::Search::EXPORT = qw(IsValidQueryType);
 
 use Bugzilla::Config;
 use Bugzilla::Error;
@@ -99,7 +101,12 @@ sub init {
         push @supptables, "components AS map_components";
         push @wherepart, "bugs.component_id = map_components.id";
     }
-
+    
+    if (grep($_ =~ /AS (actual_time|percentage_complete)$/, @$fieldsref)) {
+        push(@supptables, "longdescs AS ldtime");
+        push(@wherepart, "ldtime.bug_id = bugs.bug_id");
+    }
+    
     my $minvotes;
     if (defined $params->param('votes')) {
         my $c = trim($params->param('votes'));
@@ -169,11 +176,6 @@ sub init {
         push(@specialchart, ["keywords", $t, $params->param('keywords')]);
     }
 
-    if (lsearch($fieldsref, "(SUM(ldtime.work_time)*COUNT(DISTINCT ldtime.bug_when)/COUNT(bugs.bug_id)) AS actual_time") != -1) {
-        push(@supptables, "longdescs AS ldtime");
-        push(@wherepart, "ldtime.bug_id = bugs.bug_id");
-    }
-
     foreach my $id ("1", "2") {
         if (!defined ($params->param("email$id"))) {
             next;
@@ -200,22 +202,7 @@ sub init {
             }
         }
         if ($params->param("emaillongdesc$id")) {
-            if (my $list = $self->ListIDsForEmail($type, $email)) {
-                my $table = "longdescs_email_$id";
-                push(@supptables, "LEFT JOIN longdescs $table ON bugs.bug_id = $table.bug_id AND $table.who IN($list)");
-                push(@wherepart, "$table.who IS NOT NULL");
-                # push something into @clist so that we don't trigger
-                # the missing_email_type error below
-                push(@clist, 'noop');
-            } else {
-                my $table = "longdescs_email_$id";
-                push(@supptables, "longdescs $table");
-                push(@wherepart, "$table.bug_id = bugs.bug_id");
-                my $ptable = "longdescnames_email_$id";
-                push(@supptables, "profiles $ptable");
-                push(@wherepart, "$table.who = $ptable.userid");
-                push(@clist, "$ptable.login_name", $type, $email);
-            }
+                push(@clist, "commenter", $type, $email);
         }
         if (@clist) {
             push(@specialchart, \@clist);
@@ -273,8 +260,8 @@ sub init {
                     # Treat [Bug creation] differently because we need to look
                     # at bugs.creation_ts rather than the bugs_activity table.
                     my @l;
-                    push(@l, "creation_ts >= $sql_chfrom") if($sql_chfrom);
-                    push(@l, "creation_ts <= $sql_chto") if($sql_chto);
+                    push(@l, "bugs.creation_ts >= $sql_chfrom") if($sql_chfrom);
+                    push(@l, "bugs.creation_ts <= $sql_chto") if($sql_chto);
                     $bug_creation_clause = "(" . join(' AND ', @l) . ")";
                 } else {
                     push(@list, "\nactcheck.fieldid = " . &::GetFieldID($f));
@@ -321,7 +308,15 @@ sub init {
     }
 
     if (defined $params->param('content')) {
-        push(@specialchart, ['content', 'matches', $params->param('content')]);
+        # Append a new chart implementing content quicksearch
+        my $chart;
+        for ($chart = 0 ; $params->param("field$chart-0-0") ; $chart++) {};
+        $params->param("field$chart-0-0", 'content');
+        $params->param("type$chart-0-0", 'matches');
+        $params->param("value$chart-0-0", $params->param('content'));
+        $params->param("field$chart-0-1", 'short_desc');
+        $params->param("type$chart-0-1", 'allwords');
+        $params->param("value$chart-0-1", $params->param('content'));
     }
 
     my $chartid;
@@ -429,12 +424,9 @@ sub init {
              # The term to use in the WHERE clause.
              $term = $term1;
 
-             # In order to sort by relevance, we SELECT the relevance value
-             # and give it an alias so we can add it to the SORT BY clause
-             # when we build that clause in buglist.cgi.  We also flag the
-             # query in Bugzilla with the "sorted_by_relevance" flag
-             # so buglist.cgi knows to sort by relevance instead of anything
-             # else the user selected.
+             # In order to sort by relevance (in case the user requests it),
+             # we SELECT the relevance value and give it an alias so we can
+             # add it to the SORT BY clause when we build it in buglist.cgi.
              #
              # Note: MySQL calculates relevance for each comment separately,
              # so we need to do some additional calculations to get an overall
@@ -446,8 +438,47 @@ sub init {
              # Note: We should be calculating the average relevance of all
              # comments for a bug, not just matching comments, but that's hard
              # (see http://bugzilla.mozilla.org/show_bug.cgi?id=145588#c35).
-             push(@fields, "(SUM($term1)/COUNT($term1) + $term2) AS relevance");
-             $self->{'sorted_by_relevance'} = 1;
+             my $select_term =
+               "(SUM($term1)/COUNT($term1) + $term2) AS relevance";
+
+             # Users can specify to display the relevance field, in which case
+             # it'll show up in the list of fields being selected, and we need
+             # to replace that occurrence with our select term.  Otherwise
+             # we can just add the term to the list of fields being selected.
+             if (grep($_ eq "relevance", @fields)) {
+                 @fields = map($_ eq "relevance" ? $select_term : $_ , @fields);
+             }
+             else {
+                 push(@fields, $select_term);
+             }
+         },
+         "^content," => sub {
+             ThrowUserError("search_content_without_matches");
+         },
+         "^commenter," => sub {    
+             my $chartseq;
+             my $list;
+             $list = $self->ListIDsForEmail($t, $v);
+             $chartseq = $chartid;
+             if ($chartid eq "") {
+                 $chartseq = "LD$sequence";
+                 $sequence++;
+             }
+             my $table = "longdescs_$chartseq";
+             my $extra = "";
+             if (Param("insidergroup") && !&::UserInGroup(Param("insidergroup"))) {
+                 $extra = "AND $table.isprivate < 1";
+             }
+             if ($list) {
+                 push(@supptables, "LEFT JOIN longdescs $table ON $table.bug_id = bugs.bug_id $extra AND $table.who IN ($list)");
+                 $term = "$table.who IS NOT NULL";
+             } else {
+                 push(@supptables, "LEFT JOIN longdescs $table ON $table.bug_id = bugs.bug_id $extra");
+                 push(@supptables, "LEFT JOIN profiles map_$table ON $table.who = map_$table.userid");
+                 $ff = $f = "map_$table.login_name";
+                 my $ref = $funcsbykey{",$t"};
+                 &$ref;
+             }
          },
          "^long_?desc," => sub {
              my $table = "longdescs_$chartid";
@@ -568,7 +599,8 @@ sub init {
              # negative conditions (f.e. "flag isn't review+").
              my $flags = "flags_$chartid";
              push(@supptables, "LEFT JOIN flags $flags " . 
-                               "ON bugs.bug_id = $flags.bug_id");
+                               "ON bugs.bug_id = $flags.bug_id " .
+                               "AND $flags.is_active = 1");
              my $flagtypes = "flagtypes_$chartid";
              push(@supptables, "LEFT JOIN flagtypes $flagtypes " . 
                                "ON $flags.type_id = $flagtypes.id");
@@ -593,18 +625,22 @@ sub init {
                 $term = "0=0";
              }
          },
-         "^requesters.login_name," => sub {
-             push(@supptables, "flags flags_$chartid");
-             push(@wherepart, "bugs.bug_id = flags_$chartid.bug_id");
-             push(@supptables, "profiles requesters_$chartid");
-             push(@wherepart, "flags_$chartid.requester_id = requesters_$chartid.userid");
-             $f = "requesters_$chartid.login_name";
+         "^requestees.login_name," => sub {
+             my $flags = "flags_$chartid";
+             push(@supptables, "LEFT JOIN flags $flags " .
+                               "ON bugs.bug_id = $flags.bug_id " .
+                               "AND $flags.is_active = 1");
+             push(@supptables, "LEFT JOIN profiles requestees_$chartid " .
+                               "ON $flags.requestee_id = requestees_$chartid.userid");
+             $f = "requestees_$chartid.login_name";
          },
          "^setters.login_name," => sub {
-             push(@supptables, "flags flags_$chartid");
-             push(@wherepart, "bugs.bug_id = flags_$chartid.bug_id");
-             push(@supptables, "profiles setters_$chartid");
-             push(@wherepart, "flags_$chartid.setter_id = setters_$chartid.userid");
+             my $flags = "flags_$chartid";
+             push(@supptables, "LEFT JOIN flags $flags " .
+                               "ON bugs.bug_id = $flags.bug_id " .
+                               "AND $flags.is_active = 1");
+             push(@supptables, "LEFT JOIN profiles setters_$chartid " .
+                               "ON $flags.setter_id = setters_$chartid.userid");
              $f = "setters_$chartid.login_name";
          },
          
@@ -685,6 +721,42 @@ sub init {
                 push(@wherepart, "$table.dependson = bugs.bug_id");
          },
 
+         "^owner_idle_time,(greaterthan|lessthan)" => sub {
+                my $table = "idle_" . $chartid;
+                $v =~ /^(\d+)\s*([hHdDwWmMyY])?$/;
+                my $quantity = $1;
+                my $unit = lc $2;
+                my $unitinterval = 'DAY';
+                if ($unit eq 'h') {
+                    $unitinterval = 'HOUR';
+                } elsif ($unit eq 'w') {
+                    $unitinterval = ' * 7 DAY';
+                } elsif ($unit eq 'm') {
+                    $unitinterval = 'MONTH';
+                } elsif ($unit eq 'y') {
+                    $unitinterval = 'YEAR';
+                }
+                my $cutoff = "DATE_SUB(NOW(), 
+                              INTERVAL $quantity $unitinterval)";
+                my $assigned_fieldid = &::GetFieldID('assigned_to');
+                push(@supptables, "LEFT JOIN longdescs comment_$table " .
+                                  "ON comment_$table.who = bugs.assigned_to " .
+                                  "AND comment_$table.bug_id = bugs.bug_id " .
+                                  "AND comment_$table.bug_when > $cutoff");
+                push(@supptables, "LEFT JOIN bugs_activity activity_$table " .
+                                  "ON (activity_$table.who = bugs.assigned_to " .
+                                  "OR activity_$table.fieldid = $assigned_fieldid) " .
+                                  "AND activity_$table.bug_id = bugs.bug_id " .
+                                  "AND activity_$table.bug_when > $cutoff");
+                if ($t =~ /greater/) {
+                    push(@wherepart, "(comment_$table.who IS NULL " .
+                                     "AND activity_$table.who IS NULL)");
+                } else {
+                    push(@wherepart, "(comment_$table.who IS NOT NULL " .
+                                     "OR activity_$table.who IS NOT NULL)");
+                }
+                $term = "0=0";
+         },
 
          ",equals" => sub {
              $term = "$ff = $q";
@@ -693,7 +765,15 @@ sub init {
              $term = "$ff != $q";
          },
          ",casesubstring" => sub {
-             $term = "INSTR(CAST($ff AS BINARY), CAST($q AS BINARY))";
+             # mysql 4.0.1 and lower do not support CAST
+             # mysql 3.*.* had a case-sensitive INSTR
+             # (checksetup has a check for unsupported versions)
+             my $server_version = Bugzilla::DB->server_version;
+             if ($server_version =~ /^3\./) {
+                 $term = "INSTR($ff ,$q)";
+             } else {
+                 $term = "INSTR(CAST($ff AS BINARY), CAST($q AS BINARY))";
+             }
          },
          ",substring" => sub {
              $term = "INSTR(LOWER($ff), " . lc($q) . ")";
@@ -713,6 +793,9 @@ sub init {
          ",lessthan" => sub {
              $term = "$ff < $q";
          },
+         ",matches" => sub {
+             ThrowUserError("search_content_without_matches");
+         },
          ",greaterthan" => sub {
              $term = "$ff > $q";
          },
@@ -917,6 +1000,8 @@ sub init {
 # @supptables = Tables and/or table aliases used in query
 # %suppseen   = A hash used to store all the tables in supptables to weed
 #               out duplicates.
+# @supplist   = A list used to accumulate all the JOIN clauses for each
+#               chart to merge the ON sections of each.
 # $suppstring = String which is pasted into query containing all table names
 
     # get a list of field names to verify the user-submitted chart fields against
@@ -998,15 +1083,25 @@ sub init {
     }
     my %suppseen = ("bugs" => 1);
     my $suppstring = "bugs";
+    my @supplist = (" ");
     foreach my $str (@supptables) {
         if (!$suppseen{$str}) {
-            if ($str !~ /^(LEFT|INNER) JOIN/i) {
-                $suppstring .= ",";
+            if ($str =~ /^(LEFT|INNER) JOIN/i) {
+                $str =~ /^(.*?)\s+ON\s+(.*)$/i;
+                my ($leftside, $rightside) = ($1, $2);
+                if ($suppseen{$leftside}) {
+                    $supplist[$suppseen{$leftside}] .= " AND ($rightside)";
+                } else {
+                    $suppseen{$leftside} = scalar @supplist;
+                    push @supplist, " $leftside ON ($rightside)";
+                }
+            } else {
+                $suppstring .= ", $str";
+                $suppseen{$str} = 1;
             }
-            $suppstring .= " $str";
-            $suppseen{$str} = 1;
         }
     }
+    $suppstring .= join('', @supplist);
     
     # Make sure we create a legal SQL query.
     @andlist = ("1 = 1") if !@andlist;
@@ -1025,7 +1120,7 @@ sub init {
     }
 
     $query .= " WHERE " . join(' AND ', (@wherepart, @andlist)) .
-              " AND ((bug_group_map.group_id IS NULL)";
+              " AND bugs.creation_ts IS NOT NULL AND ((bug_group_map.group_id IS NULL)";
 
     if ($user) {
         my $userid = $user->id;
@@ -1092,7 +1187,7 @@ sub SqlifyDate {
 # ListIDsForEmail returns a string with a comma-joined list
 # of userids matching email addresses
 # according to the type specified.
-# Currently, this only supports anyexact and substring matches.
+# Currently, this only supports exact, anyexact, and substring matches.
 # Substring matches will return up to 50 matching userids
 # If a match type is unsupported or returns too many matches,
 # ListIDsForEmail returns an undef.
@@ -1179,4 +1274,14 @@ sub getSQL {
     return $self->{'sql'};
 }
 
+# Define if the Query Type passed in is a valid query type that we can deal with
+sub IsValidQueryType
+{
+    my ($queryType) = @_;
+    if (grep { $_ eq $queryType } qw(specific advanced)) {
+        return 1;
+    }
+    return 0;
+}
+
 1;
