@@ -91,7 +91,7 @@ sub getTemplateIncludePath () {
         # 'en-uk' etc. but not the otherway round. (This is unfortunally
         # not very clearly stated in those RFC; see comment just over 14.5
         # in http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.4)
-        if(my @found = grep /^$lang(-.+)?$/i, @languages) {
+        if(my @found = grep /^\Q$lang\E(-.+)?$/i, @languages) {
             push (@usedlanguages, @found);
         }
     }
@@ -195,14 +195,41 @@ sub create {
         # built-in filter, please also add a stub filter to checksetup.pl
         # and t/004template.t.
         FILTERS => {
-            # Render text in strike-through style.
-            strike => sub { return "<strike>" . $_[0] . "</strike>" },
+
+            # Render text in required style.
+
+            inactive => [
+                sub {
+                    my($context, $isinactive) = @_;
+                    return sub {
+                        return $isinactive ? '<span class="bz_inactive">'.$_[0].'</span>' : $_[0];
+                    }
+                }, 1
+            ],
+
+            closed => [
+                sub {
+                    my($context, $isclosed) = @_;
+                    return sub {
+                        return $isclosed ? '<span class="bz_closed">'.$_[0].'</span>' : $_[0];
+                    }
+                }, 1
+            ],
+
+            obsolete => [
+                sub {
+                    my($context, $isobsolete) = @_;
+                    return sub {
+                        return $isobsolete ? '<span class="bz_obsolete">'.$_[0].'</span>' : $_[0];
+                    }
+                }, 1
+            ],
 
             # Returns the text with backslashes, single/double quotes,
             # and newlines/carriage returns escaped for use in JS strings.
             js => sub {
                 my ($var) = @_;
-                $var =~ s/([\\\'\"])/\\$1/g;
+                $var =~ s/([\\\'\"\/])/\\$1/g;
                 $var =~ s/\n/\\n/g;
                 $var =~ s/\r/\\r/g;
                 $var =~ s/\@/\\x40/g; # anti-spam for email addresses
@@ -293,6 +320,31 @@ sub create {
                 return $var;
             },
             
+            # iCalendar contentline filter
+            ics => [ sub {
+                         my ($context, @args) = @_;
+                         return sub {
+                             my ($var) = shift;
+                             my ($par) = shift @args;
+                             my ($output) = "";
+
+                             $var =~ s/[\r\n]/ /g;
+                             $var =~ s/([;\\\"])/\\$1/g;
+
+                             if ($par) {
+                                 $output = sprintf("%s:%s", $par, $var);
+                             } else {
+                                 $output = $var;
+                             }
+                             
+                             $output =~ s/(.{75,75})/$1\n /g;
+
+                             return $output;
+                         };
+                     },
+                     1
+                     ],
+
             # We force filtering of every variable in key security-critical
             # places; we have a none filter for people to use when they 
             # really, really don't want a variable to be changed.
