@@ -35,6 +35,7 @@ package Bugzilla::User;
 use Bugzilla::Config;
 use Bugzilla::Error;
 use Bugzilla::Util;
+use Bugzilla::Constants;
 
 ################################################################################
 # Functions
@@ -107,7 +108,14 @@ sub _create {
                                        $id);
 
     if ($result) {
+        my $is_main_db;
+        unless ($is_main_db = Bugzilla->dbwritesallowed()) {
+            Bugzilla->switch_to_main_db();
+        }
         $self->derive_groups($tables_locked_for_derive_groups);
+        unless ($is_main_db) {
+            Bugzilla->switch_to_shadow_db();
+        }
     }
 
     return $self;
@@ -116,12 +124,12 @@ sub _create {
 # Accessors for user attributes
 sub id { $_[0]->{id}; }
 sub login { $_[0]->{login}; }
-sub email { $_[0]->{login}; }
+sub email { $_[0]->{login} . Param('emailsuffix'); }
 sub name { $_[0]->{name}; }
 sub showmybugslink { $_[0]->{showmybugslink}; }
 
-# Generate a string to identify the user by name + email if the user
-# has a name or by email only if she doesn't.
+# Generate a string to identify the user by name + login if the user
+# has a name or by login only if she doesn't.
 sub identity {
     my $self = shift;
 
@@ -206,7 +214,7 @@ sub in_group {
 
     my $dbh = Bugzilla->dbh;
 
-    my $res = $dbh->selectrow(q{SELECT 1
+    my ($res) = $dbh->selectrow_array(q{SELECT 1
                                   FROM groups, user_group_map
                                  WHERE groups.id=user_group_map.group_id
                                    AND user_group_map.user_id=?
@@ -239,9 +247,10 @@ sub derive_groups {
     # first remove any old derived stuff for this user
     $dbh->do(q{DELETE FROM user_group_map
                       WHERE user_id = ?
-                        AND isderived = 1},
+                        AND grant_type != ?},
              undef,
-             $id);
+             $id,
+             GRANT_DIRECT);
 
     my %groupidsadded = ();
     # add derived records for any matching regexps
@@ -253,10 +262,10 @@ sub derive_groups {
     while (my $row = $sth->fetch) {
         if ($self->{login} =~ m/$row->[1]/i) {
             $group_insert ||= $dbh->prepare(q{INSERT INTO user_group_map
-                                              (user_id, group_id, isbless, isderived)
-                                              VALUES (?, ?, 0, 1)});
+                                              (user_id, group_id, isbless, grant_type)
+                                              VALUES (?, ?, 0, ?)});
             $groupidsadded{$row->[0]} = 1;
-            $group_insert->execute($id, $row->[0]);
+            $group_insert->execute($id, $row->[0], GRANT_REGEXP);
         }
     }
 
@@ -287,9 +296,9 @@ sub derive_groups {
                 if (!$groupidsadded{$groupid}) {
                     $groupidsadded{$groupid} = 1;
                     $group_insert ||= $dbh->prepare(q{INSERT INTO user_group_map
-                                                      (user_id, group_id, isbless, isderived)
-                                                      VALUES (?, ?, 0, 1)});
-                    $group_insert->execute($id, $groupid);
+                                                      (user_id, group_id, isbless, grant_type)
+                                                      VALUES (?, ?, 0, ?)});
+                    $group_insert->execute($id, $groupid, GRANT_DERIVED);
                 }
             }
         }
@@ -657,20 +666,6 @@ sub email_prefs {
     my @reasons = qw(Removeme Comments Attachments Status Resolved Keywords 
                      CC Other Unconfirmed);
 
-    # If the prefs are empty, this user hasn't visited the email pane
-    # of userprefs.cgi since before the change to use the "emailflags" 
-    # column, so initialize that field with the default prefs.
-    if (!$flags) {
-        # Create a default prefs string that causes the user to get all email.
-        $flags = "ExcludeSelf~on~FlagRequestee~on~FlagRequester~on~";
-        foreach my $role (@roles) {
-            foreach my $reason (@reasons) {
-                $flags .= "email$role$reason~on~";
-            }
-        }
-        chop $flags;
-    }
-
     # Convert the prefs from the flags string from the database into
     # a Perl record.  The 255 param is here because split will trim 
     # any trailing null fields without a third param, which causes Perl 
@@ -703,6 +698,17 @@ sub email_prefs {
     return $self->{email_prefs};
 }
 
+sub is_mover {
+    my $self = shift;
+
+    if (!defined $self->{'is_mover'}) {
+        my @movers = map { trim($_) } split(',', Param('movers'));
+        $self->{'is_mover'} = ($self->id
+                               && lsearch(\@movers, $self->login) != -1);
+    }
+    return $self->{'is_mover'};
+}
+
 1;
 
 __END__
@@ -855,6 +861,12 @@ all MySQL supported, this will go away.
 
 Returns C<1> if the user can bless at least one group. Otherwise returns C<0>.
 
+=item C<is_mover>
+
+Returns true if the user is in the list of users allowed to move bugs
+to another database. Note that this method doesn't check whether bug
+moving is enabled.
+
 =back
 
 =head1 SEE ALSO
