@@ -29,6 +29,7 @@ use strict;
 
 use base qw(Exporter);
 @Bugzilla::Util::EXPORT = qw(is_tainted trick_taint detaint_natural
+                             detaint_signed
                              html_quote url_quote value_quote xml_quote
                              css_class_quote
                              lsearch max min
@@ -49,14 +50,24 @@ sub is_tainted {
 sub trick_taint {
     require Carp;
     Carp::confess("Undef to trick_taint") unless defined $_[0];
-    $_[0] =~ /^(.*)$/s;
-    $_[0] = $1;
+    my ($match) = $_[0] =~ /^(.*)$/s;
+    $_[0] = $match;
     return (defined($_[0]));
 }
 
 sub detaint_natural {
-    $_[0] =~ /^(\d+)$/;
-    $_[0] = $1;
+    my ($match) = $_[0] =~ /^(\d+)$/;
+    $_[0] = $match;
+    return (defined($_[0]));
+}
+
+sub detaint_signed {
+    my ($match) = $_[0] =~ /^([-+]?\d+)$/;
+    $_[0] = $match;
+    # Remove any leading plus sign.
+    if (defined($_[0]) && $_[0] =~ /^\+(\d+)$/) {
+        $_[0] = $1;
+    }
     return (defined($_[0]));
 }
 
@@ -149,7 +160,7 @@ sub trim {
 sub format_time {
     my ($time) = @_;
 
-    my ($year, $month, $day, $hour, $min);
+    my ($year, $month, $day, $hour, $min, $sec);
     if ($time =~ m/^\d{14}$/) {
         # We appear to have a timestamp direct from MySQL
         $year  = substr($time,0,4);
@@ -158,12 +169,13 @@ sub format_time {
         $hour  = substr($time,8,2);
         $min   = substr($time,10,2);
     }
-    elsif ($time =~ m/^(\d{4})\.(\d{2})\.(\d{2}) (\d{2}):(\d{2})(:\d{2})?$/) {
+    elsif ($time =~ m/^(\d{4})[-\.](\d{2})[-\.](\d{2}) (\d{2}):(\d{2})(:(\d{2}))?$/) {
         $year  = $1;
         $month = $2;
         $day   = $3;
         $hour  = $4;
         $min   = $5;
+        $sec   = $7;
     }
     else {
         warn "Date/Time format ($time) unrecogonzied";
@@ -171,6 +183,9 @@ sub format_time {
 
     if (defined $year) {
         $time = "$year-$month-$day $hour:$min";
+        if (defined $sec) {
+            $time .= ":$sec";
+        }
         $time .= " " . &::Param('timezone') if &::Param('timezone');
     }
     return $time;
@@ -192,6 +207,7 @@ Bugzilla::Util - Generic utility functions for bugzilla
   $rv = is_tainted($var);
   trick_taint($var);
   detaint_natural($var);
+  detaint_signed($var);
 
   # Functions for quoting
   html_quote($var);
@@ -252,6 +268,12 @@ This routine detaints a natural number. It returns a true value if the
 value passed in was a valid natural number, else it returns false. You
 B<MUST> check the result of this routine to avoid security holes.
 
+=item C<detaint_signed($num)>
+
+This routine detaints a signed integer. It returns a true value if the
+value passed in was a valid signed integer, else it returns false. You
+B<MUST> check the result of this routine to avoid security holes.
+
 =back
 
 =head2 Quoting
