@@ -25,7 +25,9 @@
 #                 Jean-Sebastien Guay <jean_seb@hybride.com>
 
 # Run me out of cron at midnight to collect Bugzilla statistics.
-
+#
+# To run new charts for a specific date, pass it in on the command line in
+# ISO (2004-08-14) format.
 
 use AnyDBM_File;
 use strict;
@@ -51,7 +53,6 @@ if (chdir("graphs")) {
     chdir("..");
 }
 
-ConnectToDatabase();
 GetVersionTable();
 
 Bugzilla->switch_to_shadow_db();
@@ -59,6 +60,7 @@ Bugzilla->switch_to_shadow_db();
 # To recreate the daily statistics,  run "collectstats.pl --regenerate" .
 my $regenerate = 0;
 if ($#ARGV >= 0 && $ARGV[0] eq "--regenerate") {
+    shift(@ARGV);
     $regenerate = 1;
 }
 
@@ -85,18 +87,27 @@ my $tend = time;
 
 CollectSeriesData();
 
-# Generate a static RDF file containing the default view of the duplicates data.
-open(CGI, "GATEWAY_INTERFACE=cmdline REQUEST_METHOD=GET QUERY_STRING=ctype=rdf ./duplicates.cgi |")
-  || die "can't fork duplicates.cgi: $!";
-open(RDF, ">$datadir/duplicates.tmp")
-  || die "can't write to $datadir/duplicates.tmp: $!";
-my $headers_done = 0;
-while (<CGI>) {
-  print RDF if $headers_done;
-  $headers_done = 1 if $_ eq "\n";
+{
+    local $ENV{'GATEWAY_INTERFACE'} = 'cmdline';
+    local $ENV{'REQUEST_METHOD'} = 'GET';
+    local $ENV{'QUERY_STRING'} = 'ctype=rdf';
+
+    my $perl = $^X;
+    trick_taint($perl);
+
+    # Generate a static RDF file containing the default view of the duplicates data.
+    open(CGI, "$perl -T duplicates.cgi |")
+        || die "can't fork duplicates.cgi: $!";
+    open(RDF, ">$datadir/duplicates.tmp")
+        || die "can't write to $datadir/duplicates.tmp: $!";
+    my $headers_done = 0;
+    while (<CGI>) {
+        print RDF if $headers_done;
+        $headers_done = 1 if $_ eq "\r\n";
+    }
+    close CGI;
+    close RDF;
 }
-close CGI;
-close RDF;
 if (-s "$datadir/duplicates.tmp") {
     rename("$datadir/duplicates.rdf", "$datadir/duplicates-old.rdf");
     rename("$datadir/duplicates.tmp", "$datadir/duplicates.rdf");
@@ -131,9 +142,9 @@ sub collect_stats {
 
         foreach my $status ('NEW', 'ASSIGNED', 'REOPENED', 'UNCONFIRMED', 'RESOLVED', 'VERIFIED', 'CLOSED') {
             if( $product eq "-All-" ) {
-                SendSQL("select count(bug_status) from bugs where bug_status='$status'");
+                SendSQL("SELECT COUNT(bug_status) FROM bugs WHERE bug_status='$status'");
             } else {
-                SendSQL("select count(bug_status) from bugs where bug_status='$status' and product_id=$product_id");
+                SendSQL("SELECT COUNT(bug_status) FROM bugs WHERE bug_status='$status' AND product_id=$product_id");
             }
 
             push @row, FetchOneColumn();
@@ -141,9 +152,9 @@ sub collect_stats {
 
         foreach my $resolution ('FIXED', 'INVALID', 'WONTFIX', 'LATER', 'REMIND', 'DUPLICATE', 'WORKSFORME', 'MOVED') {
             if( $product eq "-All-" ) {
-                SendSQL("select count(resolution) from bugs where resolution='$resolution'");
+                SendSQL("SELECT COUNT(resolution) FROM bugs WHERE resolution='$resolution'");
             } else {
-                SendSQL("select count(resolution) from bugs where resolution='$resolution' and product_id=$product_id");
+                SendSQL("SELECT COUNT(resolution) FROM bugs WHERE resolution='$resolution' AND product_id=$product_id");
             }
 
             push @row, FetchOneColumn();
@@ -170,11 +181,11 @@ FIN
 }
 
 sub calculate_dupes {
-    SendSQL("SELECT * FROM duplicates");
+    my $dbh = Bugzilla->dbh;
+    my $rows = $dbh->selectall_arrayref("SELECT dupe_of, dupe FROM duplicates");
 
     my %dupes;
     my %count;
-    my @row;
     my $key;
     my $changed = 1;
 
@@ -184,6 +195,7 @@ sub calculate_dupes {
     # so we can read it back in to do changed counters
     # First, delete it if it exists, so we don't add to the contents of an old file
     if (my @files = <$datadir/duplicates/dupes$today*>) {
+        map { trick_taint($_) } @files;
         unlink @files;
     }
    
@@ -191,9 +203,8 @@ sub calculate_dupes {
 
     # Create a hash with key "a bug number", value "bug which that bug is a
     # direct dupe of" - straight from the duplicates table.
-    while (@row = FetchSQLData()) {
-        my $dupe_of = shift @row;
-        my $dupe = shift @row;
+    foreach my $row (@$rows) {
+        my ($dupe_of, $dupe) = @$row;
         $dupes{$dupe} = $dupe_of;
     }
 
@@ -437,9 +448,7 @@ sub CollectSeriesData {
     # (days_since_epoch + series_id) % frequency = 0. So they'll run every
     # <frequency> days, but the start date depends on the series_id.
     my $days_since_epoch = int(time() / (60 * 60 * 24));
-    my $today = today_dash();
-
-    CleanupChartTables() if ($days_since_epoch % 7 == 0);
+    my $today = $ARGV[0] || today_dash();
 
     # We save a copy of the main $dbh and then switch to the shadow and get
     # that one too. Remember, these may be the same.
@@ -456,13 +465,13 @@ sub CollectSeriesData {
 
     # We prepare the insertion into the data table, for efficiency.
     my $sth = $dbh->prepare("INSERT INTO series_data " .
-                            "(series_id, date, value) " .
+                            "(series_id, series_date, series_value) " .
                             "VALUES (?, " . $dbh->quote($today) . ", ?)");
 
     # We delete from the table beforehand, to avoid SQL errors if people run
     # collectstats.pl twice on the same day.
     my $deletesth = $dbh->prepare("DELETE FROM series_data 
-                                   WHERE series_id = ? AND date = " .
+                                   WHERE series_id = ? AND series_date = " .
                                    $dbh->quote($today));
                                      
     foreach my $series_id (keys %$serieses) {
@@ -476,37 +485,23 @@ sub CollectSeriesData {
                                           'user'   => $user);
         my $sql = $search->getSQL();
         
-        # We need to count the returned rows. Without subselects, we can't
-        # do this directly in the SQL for all queries. So we do it by hand.
-        my $data = $shadow_dbh->selectall_arrayref($sql);
+        my $data;
         
-        my $count = scalar(@$data) || 0;
+        # We can't die if we get dodgy SQL back for whatever reason, so we
+        # eval() this and, if it fails, just ignore it and carry on.
+        # One day we might even log an error.
+        eval { 
+            $data = $shadow_dbh->selectall_arrayref($sql);
+        };
+        
+        if (!$@) {
+            # We need to count the returned rows. Without subselects, we can't
+            # do this directly in the SQL for all queries. So we do it by hand.
+            my $count = scalar(@$data) || 0;
 
-        $deletesth->execute($series_id);
-        $sth->execute($series_id, $count);
+            $deletesth->execute($series_id);
+            $sth->execute($series_id, $count);
+        }
     }
 }
 
-sub CleanupChartTables {
-    Bugzilla->switch_to_main_db();
-    my $dbh = Bugzilla->dbh;
-
-    $dbh->do("LOCK TABLES series WRITE, user_series_map AS usm READ");
-
-    # Find all those that no-one subscribes to
-    my $series_data = $dbh->selectall_arrayref("SELECT series.series_id " .
-                              "FROM series LEFT JOIN user_series_map AS usm " .
-                              "ON series.series_id = usm.series_id " .
-                              "WHERE usm.series_id IS NULL");
-
-    my $series_ids = join(",", map({ $_->[0] } @$series_data));
-
-    # Stop collecting data on all series which no-one is subscribed to.
-    if ($series_ids) {
-        $dbh->do("UPDATE series SET frequency = 0 " . 
-                 "WHERE series_id IN($series_ids)");
-    }
-   
-    $dbh->do("UNLOCK TABLES");
-    Bugzilla->switch_to_shadow_db();
-}
