@@ -503,13 +503,15 @@ sub find_languages {
 
   {
    name => 'sendmailnow',
-   desc => 'If this is on, Bugzilla will tell sendmail to send any e-mail ' .
-           'immediately. If you have a large number of users with a large ' .
-           'amount of e-mail traffic, enabling this option may dramatically ' .
-           'slow down Bugzilla. Best used for smaller installations of ' .
-           'Bugzilla.',
+   desc => 'Sites using anything older than version 8.12 of \'sendmail\' ' .
+           'can achieve a significant performance increase in the ' .
+           'UI -- at the cost of delaying the sending of mail -- by ' .
+           'disabling this parameter. Sites using \'sendmail\' 8.12 or ' .
+           'higher should leave this on, as they will see no benefit from ' .
+           'turning it off. Sites using an MTA other than \'sendmail\' ' .
+           '*must* leave it on, or no bug mail will be sent.',
    type => 'b',
-   default => 0
+   default => 1
   },
 
   {
@@ -568,8 +570,9 @@ Configure bugmail: %urlbase%userprefs.cgi?tab=email
 
   {
    name => 'whinedays',
-   desc => 'The number of days that we\'ll let a bug sit untouched in a NEW ' .
-           'state before our cronjob will whine at the owner.',
+   desc => q{The number of days that we'll let a bug sit untouched in a NEW
+             state before our cronjob will whine at the owner.<br>
+             Set to 0 to disable whining.},
    type => 't',
    default => 7
   },
@@ -631,8 +634,8 @@ You will get this message once a day until you\'ve dealt with these bugs!
   {
    name => 'defaultquery',
    desc => 'This is the default query that initially comes up when you ' .
-           'submit a bug.  It\'s in URL parameter format, which makes it ' .
-           'hard to read.  Sorry!',
+           'access the advanced query page.  It\'s in URL parameter ' .
+           'format, which makes it hard to read.  Sorry!',
    type => 't',
    default => 'bug_status=NEW&bug_status=ASSIGNED&bug_status=REOPENED&emailassigned_to1=1&emailassigned_to2=1&emailreporter2=1&emailcc2=1&emailqa_contact2=1&order=Importance&long_desc_type=substring'
   },
@@ -1034,6 +1037,17 @@ Reason: %reason%
    checker => \&check_numeric
   },
 
+  {
+   name => 'chartgroup',
+   desc => 'The name of the group of users who can use the "New Charts" ' .
+           'feature. Administrators should ensure that the public categories ' .
+           'and series definitions do not divulge confidential information ' .
+           'before enabling this for an untrusted population. If left blank, ' .
+           'no users will be able to use New Charts.',
+   type => 't',
+   default => 'editbugs'
+  },
+  
   {
    name => 'insidergroup',
    desc => 'The name of the group of users who can see/change private ' .
@@ -1169,7 +1183,14 @@ Reason: %reason%
    type    => 't',
    default => '',
   },
-);
 
+  {
+   name    => 'noresolveonopenblockers',
+   desc    => 'Don\'t allow bugs to be resolved as fixed if they have unresolved dependencies.',
+   type    => 'b',
+   default => 0,
+  },
+  
+);
 1;
 
