@@ -33,6 +33,7 @@ use Bugzilla::Constants;
 use Bugzilla::Util;
 # Bring ChmodDataFile in until this is all moved to the module
 use Bugzilla::Config qw(:DEFAULT ChmodDataFile $localconfig $datadir);
+use Bugzilla::BugMail;
 
 # Shut up misguided -w warnings about "used only once".  For some reason,
 # "use vars" chokes on me when I try it here.
@@ -74,7 +75,7 @@ use DBI;
 
 use Date::Format;               # For time2str().
 use Date::Parse;               # For str2time().
-use RelationSet;
+use Bugzilla::RelationSet;
 
 # Use standard Perl libraries for cross-platform file/directory manipulation.
 use File::Spec;
@@ -115,23 +116,19 @@ $::unconfirmedstate = "UNCONFIRMED";
 sub AppendComment {
     my ($bugid, $who, $comment, $isprivate, $timestamp, $work_time) = @_;
     $work_time ||= 0;
-    
+
+    if ($work_time) {
+        require Bugzilla::Bug;
+        Bugzilla::Bug::ValidateTime($work_time, "work_time");
+    }
+
     # Use the date/time we were given if possible (allowing calling code
     # to synchronize the comment's timestamp with those of other records).
     $timestamp = ($timestamp ? SqlQuote($timestamp) : "NOW()");
-    
+
     $comment =~ s/\r\n/\n/g;     # Get rid of windows-style line endings.
     $comment =~ s/\r/\n/g;       # Get rid of mac-style line endings.
 
-    # allowing negatives though so people can back out errors in time reporting
-    if (defined $work_time) {
-       # regexp verifies one or more digits, optionally followed by a period and
-       # zero or more digits, OR we have a period followed by one or more digits
-       if ($work_time !~ /^-?(?:\d+(?:\.\d*)?|\.\d+)$/) { 
-          ThrowUserError("need_numeric_value");
-       }
-    } else { $work_time = 0 };
-
     if ($comment =~ /^\s*$/) {  # Nothin' but whitespace
         return;
     }
@@ -354,8 +351,8 @@ sub GetVersionTable {
         $mtime = 0;
     }
     if (time() - $mtime > 3600) {
-        use Token;
-        Token::CleanTokenTable() if Bugzilla->dbwritesallowed;
+        use Bugzilla::Token;
+        Bugzilla::Token::CleanTokenTable() if Bugzilla->dbwritesallowed;
         GenerateVersionTable();
     }
     require "$datadir/versioncache";
@@ -411,13 +408,15 @@ sub InsertNewUser {
     my $cryptpassword = Crypt($password);
 
 
+    my $defaultflagstring = SqlQuote(Bugzilla::Constants::DEFAULT_EMAIL_SETTINGS); 
+
     # Insert the new user record into the database.            
     $username = SqlQuote($username);
     $realname = SqlQuote($realname);
     $cryptpassword = SqlQuote($cryptpassword);
     PushGlobalSQLState();
-    SendSQL("INSERT INTO profiles (login_name, realname, cryptpassword) 
-             VALUES ($username, $realname, $cryptpassword)");
+    SendSQL("INSERT INTO profiles (login_name, realname, cryptpassword, emailflags) 
+             VALUES ($username, $realname, $cryptpassword, $defaultflagstring)");
     PopGlobalSQLState();
 
     # Return the password to the calling code so it can be included 
@@ -425,42 +424,9 @@ sub InsertNewUser {
     return $password;
 }
 
-# Removes all entries from logincookies for $userid, except for the
-# optional $keep, which refers the logincookies.cookie primary key.
-# (This is useful so that a user changing their password stays logged in)
-sub InvalidateLogins {
-    my ($userid, $keep) = @_;
-
-    my $remove = "DELETE FROM logincookies WHERE userid = $userid";
-    if (defined $keep) {
-        $remove .= " AND cookie != " . SqlQuote($keep);
-    }
-    SendSQL($remove);
-}
-
 sub GenerateRandomPassword {
-    my ($size) = @_;
-
-    # Generated passwords are eight characters long by default.
-    $size ||= 8;
-
-    # The list of characters that can appear in a randomly generated password.
-    # Note that users can put any character into a password they choose
-    # themselves.
-    my @pwchars = (0..9, 'A'..'Z', 'a'..'z', '-', '_', '!', '@', '#', '$',
-        '%', '^', '*');
-
-    # The number of characters in the list.
-    my $pwcharslen = scalar(@pwchars);
-
-    # Generate the password.
-    my $password = "";
-    for ( my $i=0 ; $i<$size ; $i++ ) {
-        $password .= $pwchars[rand($pwcharslen)];
-    }
-
-    # Return the password.
-    return $password;
+    my $size = (shift or 10); # default to 10 chars if nothing specified
+    return join("", map{ ('0'..'9','a'..'z','A'..'Z')[rand 62] } (1..$size));
 }
 
 #
@@ -527,11 +493,16 @@ sub CanEditProductId {
     return (!defined($result));
 }
 
-#
-# This function determines if a user can enter bugs in the named
-# product.
+# This function determines whether or not a user can enter
+# bugs into the named product.
 sub CanEnterProduct {
-    my ($productname) = @_;
+    my ($productname, $verbose) = @_;
+    my $dbh = Bugzilla->dbh;
+
+    return unless defined($productname);
+    trick_taint($productname);
+
+    # First check whether or not the user has access to that product.
     my $query = "SELECT group_id IS NULL " .
                 "FROM products " .
                 "LEFT JOIN group_control_map " .
@@ -541,12 +512,62 @@ sub CanEnterProduct {
         $query .= "AND group_id NOT IN(" . 
                    join(',', values(%{Bugzilla->user->groups})) . ") ";
     }
-    $query .= "WHERE products.name = " . SqlQuote($productname) . " LIMIT 1";
-    PushGlobalSQLState();
-    SendSQL($query);
-    my ($ret) = FetchSQLData();
-    PopGlobalSQLState();
-    return ($ret);
+    $query .= "WHERE products.name = ? LIMIT 1";
+
+    my $has_access = $dbh->selectrow_array($query, undef, $productname);
+    if (!$has_access) {
+        # Do we require the exact reason why we cannot enter
+        # bugs into that product? Returning -1 explicitely
+        # means the user has no access to the product or the
+        # product does not exist.
+        return (defined($verbose)) ? -1 : 0;
+    }
+
+    # Check if the product is open for new bugs and has
+    # at least one component.
+    my $allow_new_bugs =
+        $dbh->selectrow_array("SELECT CASE WHEN disallownew = 0 THEN 1 ELSE 0 END
+                               FROM products INNER JOIN components
+                               ON components.product_id = products.id
+                               WHERE products.name = ? LIMIT 1",
+                               undef, $productname);
+
+    # Return 1 if the user can enter bugs into that product;
+    # return 0 if the product is closed for new bug entry;
+    # return undef if the product has no component.
+    return $allow_new_bugs;
+}
+
+# Call CanEnterProduct() and display an error message
+# if the user cannot enter bugs into that product.
+sub CanEnterProductOrWarn {
+    my ($product) = @_;
+
+    if (!defined($product)) {
+        ThrowUserError("no_products");
+    }
+    my $status = CanEnterProduct($product, 1);
+    trick_taint($product);
+
+    if (!defined($status)) {
+        ThrowUserError("no_components", { product => $product});
+    } elsif (!$status) {
+        ThrowUserError("product_disabled", { product => $product});
+    } elsif ($status < 0) {
+        ThrowUserError("entry_access_denied", { product => $product});
+    }
+    return $status;
+}
+
+sub GetEnterableProducts {
+    my @products;
+    # XXX rewrite into pure SQL instead of relying on legal_products?
+    foreach my $p (@::legal_product) {
+        if (CanEnterProduct($p)) {
+            push @products, $p;
+        }
+    }
+    return (@products);
 }
 
 #
@@ -606,21 +627,23 @@ sub GetSelectableProductHash {
 
     PushGlobalSQLState();
     foreach my $table (@tables) {
-        # Why oh why can't we standardize on these names?!?
-        my $fld = ($table eq "components" ? "name" : "value");
-
-        my $query = "SELECT $fld, product_id FROM $table WHERE product_id IN " .
-                    "(" . join(",", keys %products) . ") ORDER BY $fld";
-        SendSQL($query);
-
         my %values;
         my %values_by_product;
 
-        while (MoreSQLData()) {
-            my ($name, $product_id) = FetchSQLData();
-            next unless $name;
-            $values{$name} = 1;
-            push @{$values_by_product{$products{$product_id}}}, $name;
+        if (scalar(keys %products)) {
+            # Why oh why can't we standardize on these names?!?
+            my $fld = ($table eq "components" ? "name" : "value");
+
+            my $query = "SELECT $fld, product_id FROM $table WHERE product_id " .
+                        "IN (" . join(",", keys %products) . ") ORDER BY $fld";
+            SendSQL($query);
+
+            while (MoreSQLData()) {
+                my ($name, $product_id) = FetchSQLData();
+                next unless $name;
+                $values{$name} = 1;
+                push @{$values_by_product{$products{$product_id}}}, $name;
+            }
         }
 
         $selectables->{"legal_$table"} = [sort keys %values];
@@ -635,13 +658,14 @@ sub GetSelectableProductHash {
 sub GetFieldDefs {
     my $extra = "";
     if (!UserInGroup(Param('timetrackinggroup'))) {
-        $extra = "WHERE name NOT IN ('estimated time', 'remaining_time', " .
+        $extra = "AND name NOT IN ('estimated_time', 'remaining_time', " .
                  "'work_time', 'percentage_complete')";
     }
 
     my @fields;
     PushGlobalSQLState();
-    SendSQL("SELECT name, description FROM fielddefs $extra ORDER BY sortkey");
+    SendSQL("SELECT name, description FROM fielddefs " .
+            "WHERE obsolete = 0 $extra ORDER BY sortkey");
     while (MoreSQLData()) {
         my ($name, $description) = FetchSQLData();
         push(@fields, { name => $name, description => $description });
@@ -674,7 +698,8 @@ sub CanSeeBug {
         " user_group_map.group_id = bug_group_map.group_id" .
         " AND user_group_map.isbless = 0" .
         " AND user_group_map.user_id = $userid" .
-        " WHERE bugs.bug_id = $id GROUP BY bugs.bug_id";
+        " WHERE bugs.bug_id = $id AND creation_ts IS NOT NULL" .
+        " GROUP BY bugs.bug_id";
     PushGlobalSQLState();
     SendSQL($query);
     my ($found_id, $reporter, $assigned_to, $qa_contact,
@@ -682,8 +707,9 @@ sub CanSeeBug {
         $found_cc, $found_groups, $found_members) 
         = FetchSQLData();
     PopGlobalSQLState();
-    return (
-               ($found_groups == 0) 
+    return ($found_id
+            &&
+              (($found_groups == 0) 
                || (($userid > 0) && 
                   (
                        ($assigned_to == $userid) 
@@ -692,6 +718,7 @@ sub CanSeeBug {
                     || ($found_cc && $cc_access) 
                     || ($found_groups == $found_members)
                   ))
+              )
            );
 }
 
@@ -703,11 +730,11 @@ sub ValidatePassword {
     my ($password, $matchpassword) = @_;
     
     if (length($password) < 3) {
-        ThrowUserError("password_too_short");
+        ThrowUserError("password_too_short", $::vars);
     } elsif (length($password) > 16) {
-        ThrowUserError("password_too_long");
+        ThrowUserError("password_too_long", $::vars);
     } elsif ((defined $matchpassword) && ($password ne $matchpassword)) {
-        ThrowUserError("passwords_dont_match");
+        ThrowUserError("passwords_dont_match", $::vars);
     }
 }
 
@@ -801,7 +828,7 @@ sub DBNameToIdAndCheck {
     }
 
     ThrowUserError("invalid_username",
-                   { name => $name });
+                   { name => $name }, "abort");
 }
 
 
@@ -908,14 +935,16 @@ sub quoteUrls {
     $text =~ s~\b(mailto:|)?([\w\.\-\+\=]+\@[\w\-]+(?:\.[\w\-]+)+)\b
               ~<a href=\"mailto:$2\">$1$2</a>~igx;
 
-    # attachment links - handle both cases separatly for simplicity
+    # attachment links - handle both cases separately for simplicity
     $text =~ s~((?:^Created\ an\ |\b)attachment\s*\(id=(\d+)\))
-              ~GetAttachmentLink($2, $1)
+              ~($things[$count++] = GetAttachmentLink($2, $1)) &&
+               ("\0\0" . ($count-1) . "\0\0")
               ~egmx;
 
     $text =~ s~\b(attachment\s*\#?\s*(\d+))
-              ~GetAttachmentLink($2, $1)
-              ~egmx;
+              ~($things[$count++] = GetAttachmentLink($2, $1)) &&
+               ("\0\0" . ($count-1) . "\0\0")
+              ~egmxi;
 
     # This handles bug a, comment b type stuff. Because we're using /g
     # we have to do this in one pattern, and so this is semi-messy.
@@ -988,10 +1017,10 @@ sub GetAttachmentLink {
     # Now that we know we've got all the information we're gonna get, let's
     # return the link (which is the whole reason we were called :)
     my ($title, $className) = @{$::attachlink{$attachid}};
-    # $title will be undefined if the bug didn't exist in the database.
+    # $title will be undefined if the attachment didn't exist in the database.
     if (defined $title) {
-        my $linkval = "attachment.cgi?id=$attachid&amp;action=view";
-        return qq{<a href="$linkval" class="$className" title="$title">$link_text</a>};
+        my $linkval = "attachment.cgi?id=$attachid";
+        return qq{<a href="$linkval" class="$className" title="$title">$link_text</a> [<a href="$linkval&amp;action=edit">edit</a>]};
     }
     else {
         return qq{$link_text};
@@ -1007,8 +1036,11 @@ sub GetAttachmentLink {
 
 sub GetBugLink {
     my ($bug_num, $link_text, $comment_num) = @_;
-    $bug_num || return "&lt;missing bug number&gt;";
-    detaint_natural($bug_num) || return "&lt;invalid bug number&gt;";
+    if (! defined $bug_num || $bug_num eq "") {
+        return "&lt;missing bug number&gt;";
+    }
+    my $quote_bug_num = html_quote($bug_num);
+    detaint_natural($bug_num) || return "&lt;invalid bug number: $quote_bug_num&gt;";
 
     # If we've run GetBugLink() for this bug number before, %::buglink
     # will contain an anonymous array ref of relevent values, if not
@@ -1034,9 +1066,9 @@ sub GetBugLink {
                 $post = "</i>";
             }
             elsif (! IsOpenedState($bug_state)) {
-                $pre = "<strike>";
+                $pre = '<span class="bz_closed">';
                 $title .= " $bug_res";
-                $post = "</strike>";
+                $post = '</span>';
             }
             if (CanSeeBug($bug_num, $::userid)) {
                 $title .= " - $bug_desc";
@@ -1069,6 +1101,38 @@ sub GetBugLink {
     }
 }
 
+# CountOpenDependencies counts the number of open dependent bugs for a
+# list of bugs and returns a list of bug_id's and their dependency count
+# It takes one parameter:
+#  - A list of bug numbers whose dependencies are to be checked
+
+sub CountOpenDependencies {
+    my (@bug_list) = @_;
+    my @dependencies;
+    
+    # Make sure any unfetched data from a currently running query
+    # is saved off rather than overwritten
+    PushGlobalSQLState();
+   
+    SendSQL("SELECT blocked, count(bug_status) " .
+            "FROM bugs, dependencies " .
+            "WHERE blocked IN (" . (join "," , @bug_list) . ") " .
+            "AND bug_id = dependson " .
+            "AND bug_status IN ('" . (join "','", OpenStates())  . "') " .
+            "GROUP BY blocked ");
+   
+    while (MoreSQLData()) {
+        my ($bug_id, $dependencies) = FetchSQLData();
+        push(@dependencies, { bug_id       => $bug_id, 
+                              dependencies => $dependencies });
+    }
+
+    # All done with this sidetrip
+    PopGlobalSQLState();
+
+    return @dependencies;
+}
+
 sub GetLongDescriptionAsText {
     my ($id, $start, $end) = (@_);
     my $result = "";
@@ -1176,6 +1240,10 @@ sub SplitEnumType {
 }
 
 sub UserInGroup {
+    if ($_[1]) {
+        die "UserInGroup no longer takes a second parameter.";
+    }
+    
     return defined Bugzilla->user && defined Bugzilla->user->groups->{$_[0]};
 }
 
@@ -1353,32 +1421,26 @@ sub RemoveVotes {
 
             # Now lets send the e-mail to alert the user to the fact that their votes have
             # been reduced or removed.
-            my $sendmailparm = '-ODeliveryMode=deferred';
-            if (Param('sendmailnow')) {
-               $sendmailparm = '';
-            }
-            if (open(SENDMAIL, "|/usr/lib/sendmail $sendmailparm -t -i")) {
-                my %substs;
+            my %substs;
 
-                $substs{"to"} = $name;
-                $substs{"bugid"} = $id;
-                $substs{"reason"} = $reason;
+            $substs{"to"} = $name . Param('emailsuffix');
+            $substs{"bugid"} = $id;
+            $substs{"reason"} = $reason;
 
-                $substs{"votesremoved"} = $removedvotes;
-                $substs{"votesold"} = $oldvotes;
-                $substs{"votesnew"} = $newvotes;
+            $substs{"votesremoved"} = $removedvotes;
+            $substs{"votesold"} = $oldvotes;
+            $substs{"votesnew"} = $newvotes;
 
-                $substs{"votesremovedtext"} = $removedvotestext;
-                $substs{"votesoldtext"} = $oldvotestext;
-                $substs{"votesnewtext"} = $newvotestext;
+            $substs{"votesremovedtext"} = $removedvotestext;
+            $substs{"votesoldtext"} = $oldvotestext;
+            $substs{"votesnewtext"} = $newvotestext;
 
-                $substs{"count"} = $removedvotes . "\n    " . $newvotestext;
+            $substs{"count"} = $removedvotes . "\n    " . $newvotestext;
 
-                my $msg = PerformSubsts(Param("voteremovedmail"),
-                                        \%substs);
-                print SENDMAIL $msg;
-                close SENDMAIL;
-            }
+            my $msg = PerformSubsts(Param("voteremovedmail"),
+                                    \%substs);
+
+            Bugzilla::BugMail::MessageToMTA($msg, $substs{'to'});
         }
         SendSQL("SELECT SUM(vote_count) FROM votes WHERE bug_id = $id");
         my $v = FetchOneColumn();
@@ -1399,20 +1461,20 @@ sub DiffStrings {
     my @old = split(" ", $oldstr);
     my @new = split(" ", $newstr);
 
-    my (@remove, @add) = ();
+    # For each pair of (old, new) entries: 
+    # If they're equal, set them to empty. When done, @old contains entries
+    # that were removed; @new contains ones that got added.
 
-    # Find values that were removed
-    foreach my $value (@old) {
-        push (@remove, $value) if !grep($_ eq $value, @new);
+    foreach my $oldv (@old) {
+        foreach my $newv (@new) {
+            next if ($newv eq '');
+            if ($oldv eq $newv) {
+                $newv = $oldv = '';
+            }
+        }
     }
-
-    # Find values that were added
-    foreach my $value (@new) {
-        push (@add, $value) if !grep($_ eq $value, @old);
-    }
-
-    my $removed = join (", ", @remove);
-    my $added = join (", ", @add);
+    my $removed = join (", ", grep { $_ ne '' } @old);
+    my $added = join (", ", grep { $_ ne '' } @new);
 
     return ($removed, $added);
 }
@@ -1442,7 +1504,8 @@ sub FormatTimeUnit {
 
 # Constructs a format object from URL parameters. You most commonly call it 
 # like this:
-# my $format = GetFormat("foo/bar", $::FORM{'format'}, $::FORM{'ctype'});
+# my $format = GetFormat("foo/bar", scalar($cgi->param('format')),
+#                        scalar($cgi->param('ctype')));
 
 sub GetFormat {
     my ($template, $format, $ctype) = @_;
@@ -1461,7 +1524,7 @@ sub GetFormat {
 
     # Now check that the template actually exists. We only want to check
     # if the template exists; any other errors (eg parse errors) will
-    # end up being detected laer.
+    # end up being detected later.
     eval {
         Bugzilla->template->context->template($template);
     };
