@@ -28,25 +28,28 @@ use lib qw(.);
 
 require "CGI.pl";
 
-use vars qw($template $userid %COOKIE);
+use vars qw($template $userid);
 
-use Bug;
 use Bugzilla;
+use Bugzilla::Constants;
+use Bugzilla::Bug;
 use Bugzilla::Config qw(:DEFAULT $datadir);
 use Bugzilla::BugMail;
 
 $::lockcount = 0;
 
+my $cgi = Bugzilla->cgi;
+
 unless ( Param("move-enabled") ) {
+  print $cgi->header();
+  PutHeader("Move Bugs");
   print "\n<P>Sorry. Bug moving is not enabled here. ";
   print "If you need to move a bug, contact " . Param("maintainer");
+  PutFooter();
   exit;
 }
 
-ConnectToDatabase();
-confirm_login();
-
-my $cgi = Bugzilla->cgi;
+my $user = Bugzilla->login(LOGIN_REQUIRED);
 
 sub Log {
     my ($str) = (@_);
@@ -79,7 +82,7 @@ sub Unlock {
     }
 }
 
-if ( !defined $::FORM{'buglist'} ) {
+if (!defined $cgi->param('buglist')) {
   print $cgi->header();
   PutHeader("Move Bugs");
   print "Move bugs either from the bug display page or perform a ";
@@ -90,11 +93,7 @@ if ( !defined $::FORM{'buglist'} ) {
   exit;
 }
 
-my $exporter = $::COOKIE{"Bugzilla_login"};
-my $movers = Param("movers");
-$movers =~ s/\s?,\s?/|/g;
-$movers =~ s/@/\@/g;
-unless ($exporter =~ /($movers)/) {
+unless ($user->is_mover) {
   print $cgi->header();
   PutHeader("Move Bugs");
   print "<P>You do not have permission to move bugs<P>\n";
@@ -103,14 +102,15 @@ unless ($exporter =~ /($movers)/) {
 }
 
 my @bugs;
+my $exporterid = $user->id;
 
-print "<P>\n";
-foreach my $id (split(/:/, $::FORM{'buglist'})) {
-  my $bug = new Bug($id, $::userid);
+print $cgi->header();
+PutHeader("Move Bugs");
+
+foreach my $id (split(/:/, scalar($cgi->param('buglist')))) {
+  my $bug = new Bugzilla::Bug($id, $exporterid);
   push @bugs, $bug;
   if (!$bug->error) {
-    my $exporterid = DBNameToIdAndCheck($exporter);
-
     my $fieldid = GetFieldID("bug_status");
     my $cur_status= $bug->bug_status;
     SendSQL("INSERT INTO bugs_activity " .
@@ -126,11 +126,11 @@ foreach my $id (split(/:/, $::FORM{'buglist'})) {
     SendSQL("UPDATE bugs SET resolution =\"MOVED\" where bug_id=\"$id\"");
 
     my $comment = "";
-    if (defined $::FORM{'comment'} && $::FORM{'comment'} !~ /^\s*$/) {
-        $comment .= $::FORM{'comment'} . "\n\n";
+    if (defined $cgi->param('comment') && $cgi->param('comment') !~ /^\s*$/) {
+        $comment .= $cgi->param('comment') . "\n\n";
     }
     $comment .= "Bug moved to " . Param("move-to-url") . ".\n\n";
-    $comment .= "If the move succeeded, $exporter will receive a mail\n";
+    $comment .= "If the move succeeded, " . $user->login . " will receive a mail\n";
     $comment .= "containing the number of the new bug in the other database.\n";
     $comment .= "If all went well,  please mark this bug verified, and paste\n";
     $comment .= "in a link to the new bug. Otherwise, reopen this bug.\n";
@@ -138,12 +138,11 @@ foreach my $id (split(/:/, $::FORM{'buglist'})) {
         "($id,  $exporterid, now(), " . SqlQuote($comment) . ")");
 
     print "<P>Bug $id moved to " . Param("move-to-url") . ".<BR>\n";
-    Bugzilla::BugMail::Send($id, { 'changer' => $exporter });
+    Bugzilla::BugMail::Send($id, { 'changer' => $user->login });
   }
 }
-print "<P>\n";
 
-my $buglist = $::FORM{'buglist'};
+my $buglist = $cgi->param('buglist');
 $buglist =~ s/:/,/g;
 my $host = Param("urlbase");
 $host =~ s#http://([^/]+)/.*#$1#;
@@ -155,25 +154,21 @@ $from =~ s/@/\@/;
 $msg .= "From: Bugzilla <" . $from . ">\n";
 $msg .= "Subject: Moving bug(s) $buglist\n\n";
 
-my @fieldlist = (Bug::fields(), 'group', 'long_desc', 'attachment');
+my @fieldlist = (Bugzilla::Bug::fields(), 'group', 'long_desc', 'attachment');
 my %displayfields;
 foreach (@fieldlist) {
     $displayfields{$_} = 1;
 }
 
-$template->process("bug/show.xml.tmpl", { user => { login => $exporter },
-                                          bugs => \@bugs,
+$template->process("bug/show.xml.tmpl", { bugs => \@bugs,
                                           displayfields => \%displayfields,
                                         }, \$msg)
   || ThrowTemplateError($template->error());
 
 $msg .= "\n";
 
-open(SENDMAIL,
-  "|/usr/lib/sendmail -ODeliveryMode=background -t -i") ||
-    die "Can't open sendmail";
-print SENDMAIL $msg;
-close SENDMAIL;
+Bugzilla::BugMail::MessageToMTA($msg, $to);
 
 my $logstr = "XML: bugs $buglist sent to $to";
 Log($logstr);
+PutFooter();
