@@ -174,7 +174,7 @@
 'reports/series-common.html.tmpl' => [
   'sel.name', 
   'sel.accesskey', 
-  '"onchange=\'$sel.onchange\'" IF sel.onchange', 
+  '"onchange=\"$sel.onchange\"" IF sel.onchange', 
 ],
 
 'reports/chart.csv.tmpl' => [
@@ -197,7 +197,7 @@
 'list/edit-multiple.html.tmpl' => [
   'group.id', 
   'group.description',
-  'group.description FILTER strike', 
+  'group.description FILTER inactive', 
   'knum', 
   'menuname', 
 ],
@@ -450,6 +450,7 @@
 
 'attachment/show-multiple.html.tmpl' => [
   'a.attachid', 
+  'flag.status'
 ],
 
 'attachment/updated.html.tmpl' => [
@@ -459,9 +460,8 @@
 
 'attachment/diff-header.html.tmpl' => [
   'attachid',
+  'id',
   'bugid',
-  'old_url',
-  'new_url',
   'oldid',
   'newid',
   'style',
